#include <ESPMegaPRO.h>
#include <DigitalInputCard.hpp>
#include <DigitalOutputCard.hpp>
#include <AnalogCard.hpp>

// This code demonstrates how to use the cards

uint8_t pinMap[16] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 15, 14, 13, 12};

DigitalInputCard icard = DigitalInputCard(0x21, 0x22);
DigitalOutputCard ocard = DigitalOutputCard(0x5F);

void inputCallback(uint8_t pin, bool state) {
  Serial.print("Input ");
  Serial.print(pin);
  Serial.print(" changed to ");
  Serial.println(state);
}

void printInputs() {
  for (int i = 0; i < 16; i++) {
    Serial.print("Input ");
    Serial.print(i);
    Serial.print(": ");
    Serial.print(icard.digitalRead(i));
    Serial.println();
  }
}

void setup() {
  Serial.begin(115200);
  Wire.begin(14, 33);
  // Instantiate Input Card
  Serial.println("Initializing Input Card");  
  icard.begin();
  Serial.println("Initialized Input Card");
  // Instantiate Output Card
  Serial.println("Initializing Output Card");
  ocard.begin();
  Serial.println("Initialized Output Card");

  icard.loadPinMap(pinMap);

  // Read all the inputs and print them
  printInputs();

  // Turn on all the outputs
  for (int i = 0; i < 16; i++) {
    ocard.digitalWrite(i, true);
  }

  // Set the debounce time for all inputs to 200ms
  for (int i = 0; i < 16; i++) {
    icard.setDebounceTime(i, 200);
  }

  // Register the callback function
  icard.registerCallback(inputCallback);
}

unsigned long previousMillis = 0;
const unsigned long interval = 1000; // 1 second

void loop() {
  unsigned long currentMillis = millis();

  if (currentMillis - previousMillis >= interval) {
    previousMillis = currentMillis;
    printInputs();
  }
  icard.loop();

}